@@ -228,7 +228,7 @@ class Plot(object):
     ax.semilogx()
 
     ax.set_xlabel(r"r [au]")
-    ax.set_ylabel(r"N$_\mathrm{<H>,ver}\,\mathrm{[cm^{-2}]}$")
+    ax.set_ylabel(r"N$_\mathrm{\langle H \rangle,ver}\,\mathrm{[cm^{-2}]}$")
 
     self._dokwargs(ax,**kwargs)
     self._legend(ax)
@@ -1722,7 +1722,7 @@ class Plot(object):
     ----------
 
     pseudoaniso : boolean
-      Use the pseudoe anisotropic scattering opacity.
+      Use the pseudo anisotropic scattering opacity (Default: `False`).
 
     '''
     print("PLOT: dust opacities ...")
