@@ -1375,21 +1375,25 @@ class PlotModels(object):
 
     return self._closefig(fig)
 
-  def plot_dust_opac(self,models,xenergy=False,opactype="both",**kwargs):
+  def plot_dust_opac(self,models,xenergy=False,opactype="both",pseudoaniso=False,**kwargs):
     '''
     Plots the dust opacities, absorption, scattering or extinction.
-    The pseudo anisotropic scattering is not plotted yet.
 
     Parameters
     ----------
+
     opactype : str
       `both` absorption and scattering (separately),
       `abs`  only absorption,
       `sca`  only scattering,
       `ext`  only extinction
 
+    pseudoaniso : boolean
+      Use the pseudo anisotropic scattering opacity (Default: `False`).
+
     '''
     print("PLOT: plot_dust_opac ...")
+
     fig,ax=plt.subplots(1,1,figsize=self._sfigs(**kwargs))
 
     iplot=0
@@ -1397,6 +1401,9 @@ class PlotModels(object):
     ymin=1.e100
     ymax=-1.e00
     for model in models:
+      ksca=model.dust.ksca
+      if pseudoaniso:
+        ksca=model.dust.ksca_an
 
       # plot versus energy (keV), usefull for xrays
       if xenergy:
@@ -1405,7 +1412,7 @@ class PlotModels(object):
         x=model.dust.lam
 
       if opactype=="ext":
-        y=model.dust.kabs+model.dust.ksca
+        y=model.dust.kabs+ksca
         # TODO: this is not consistent with providing the linestyle via the
         # command line
 
@@ -1433,7 +1440,7 @@ class PlotModels(object):
         if opactype=="both" or opactype=="sca":
           # TODO: this is not consistent with providing the linestyle via the
           # command line
-          y=model.dust.ksca
+          y=ksca
 
           if opactype=="both":
             col=lineabs.get_color()
