@@ -14,7 +14,8 @@ from astropy.convolution import Gaussian1DKernel
 from astropy.convolution import convolve as apy_convolve
 from scipy.constants import h,c,k
 from scipy.special import erf as serf
-
+from scipy.constants import astronomical_unit as au
+from scipy.constants import parsec as pc
 import numpy as np
 import pandas as pd
 
@@ -1252,9 +1253,9 @@ def generalConvolve(lineStrengths,lineFreq,R=1,lambda_0=1,lambda_n=1):
     FWHM=R_back/R/2.355
     g=Gaussian1DKernel(stddev=FWHM,factor=7)
     A=apy_convolve(flux_bins,g)
-    A=A[1:]/(freq_bins[:-1]-freq_bins[1:])*1e-9
+    A=A[1:]/(np.flip(freq_bins)[1:]-np.flip(freq_bins)[:-1])*1e-9
     W=np.flip(wave_bins[1:])
-    return W,A
+    return np.flip(W),np.flip(A)
 
 
 def generate_grid(R=1,lambda_0=1,lambda_n=1):
@@ -1432,3 +1433,159 @@ class MyFormatter(string.Formatter):
 
 fmt=MyFormatter()
 
+def line_flux(l1,l2,wavelength,flux):
+    '''
+    Return: Sum of integrated line flux between wavelengths l1 and l2. 
+    Units: Return is in the same unit as input. [Assumed to be in erg/s/cm2/sr]
+           l1 and l2 same unit as wavelength
+    '''
+    mask = (wavelength>l1) & (wavelength<=l2)
+    return np.sum(flux[mask])
+
+def line_fluxes(windows,wavelength,flux):
+    '''
+    Return: array of sum of integrated line flux in wavelength windows. 
+    Units: Return is in the same unit as input. [Assumed to be in erg/s/cm2/sr]
+    windows: list of wavelength limits of windows. Example: [[14,14.5],[15,16.2],[12.5,14.1]]
+    '''
+    F = []
+    for W in windows:
+        F.append(line_flux(W[0],W[1],wavelength,flux))
+    return(np.array(F))
+
+def line_flux_ratios(ratio_windows,wavelength,flux):
+    '''
+    Return: Integrated line flux ratios between wavelengths l1 and l2.
+    Unit: Return has no unit. [Input is assumed to be in erg/s/cm2/sr]
+    ratio_windows: list of wavelength windows. Example: [[[14,14.5],[15,16.2]],[[12.5,14.1],[13.4,13.8]]]
+    '''
+    R = []
+    for window in ratio_windows:
+        F = line_fluxes([window[0],window[1]],wavelength,flux)
+        R.append(F[1]/F[0])
+    return(np.array(R))
+
+def spectral_flux(l1,l2,wavelength,flux):
+    '''
+    Return: Sum of integrated line flux between wavelengths l1 and l2. 
+    Unit: Return in [erg/s/cm2] and input is in [Jy]
+          l1, l2, and wavelength in microns
+    '''
+    flux       = flux*1e-23 #Converting Jy to erg.s−1.cm−2.Hz−1
+    wavelength = wavelength*1e-4 #Converting microns to cm
+    flux       = flux[np.argsort(wavelength)]
+    wavelength = wavelength[np.argsort(wavelength)]
+    l1         = l1*1e-4 #Converting microns to cm
+    l2         = l2*1e-4 #Converting microns to cm
+    c_cms      = c*1e2 #Converting m/s to cm/s
+    mask       = (wavelength>l1) & (wavelength<=l2)
+    wave       = wavelength[mask]
+    # wave1      = np.roll(wavelength,1)[mask]
+    # wave1      = 10**((np.log10(wave)+np.log10(wave1))/2)
+    # wave2      = np.roll(wavelength,-1)[mask]
+    # wave2      = 10**((np.log10(wave)+np.log10(wave2))/2)
+    # dnu        = c*(wave2-wave1)/wave**2*1e2
+    wave1      = np.roll(wavelength,1)[mask]
+    wave2      = np.roll(wavelength,-1)[mask]
+    dnu        = (c_cms*(1/wave1-1/wave2)/2)
+    return(np.sum(flux[mask]*dnu))
+
+def spectral_fluxes(windows,wavelength,flux):
+    '''
+    Return: Sum of integrated line flux between wavelength windows.
+    Unit: return is in [erg/s/cm2] and input is in [Jy]
+    windows: list of wavelength limits (microns) of windows. Example: [[14,14.5],[15,16.2],[12.5,14.1]]
+    '''
+    F = []
+    for W in windows:
+        F.append(spectral_flux(W[0],W[1],wavelength,flux))
+    return(np.array(F))
+
+def spectral_flux_ratios(ratio_windows,wavelength,flux):
+    '''
+    Return: Spectral integrated flux ratios between wavelength windows. 
+    Units: Return is without units and input is in [Jy]
+    ratio_windows: list of wavelength (microns) windows. Example: [[[14,14.5],[15,16.2]],[[12.5,14.1],[13.4,13.8]]]
+    '''
+    R = []
+    for window in ratio_windows:
+        F = spectral_fluxes([window[0],window[1]],wavelength,flux)
+        R.append(F[1]/F[0])
+    return(np.array(R))
+
+
+def chi2_slab(slab_data,spectra,windows=[],ratio_windows=[],Rdisk=np.logspace(-2,2,10),distance=120,convolve=False,NLTE=False):
+    '''
+    Returns chi2 based on selected spectral windows across different emitting disk radius
+    slab_data: path corresponding to a single slab model or a single slab_model instance
+    spectra: numpy array with first column the wavelength in [microns] and second column flux in [Jy]
+    windows: List of spectral windows for chi2 calculation (weights optional)
+             Format: [window1,window2,....]
+             Window format: [lambda_0, lambda_1, weight]
+             weight is optional, used for weighted chi2
+             Example without weight: [[14,14.5],[15,16.2],[12.5,14.1]]
+             Example with weight: [[14,14.5,2],[15,16.2,5],[12.5,14.1,1]]
+             Weights are automatically normalised, so sum of weights need not be 1
+    Rdisk: Radius corresponding to emitting area in astronomical units
+    distance: distance of disk in parsec
+    convolve: If True, the chi2 is performed on convolved spectra and not on individual lines
+    NLTE: if True, NLTE flux is taken from the slab model.
+    '''
+    area = np.pi*(Rdisk*au/distance/pc)**2
+    chi_area = np.zeros((len(area)))
+    if isinstance(slab_data,str): slab_data = read_slab(slab_data,verbose=False)
+    if len(windows)<0 and len(ratio_windows)<0: return chi_area
+    if convolve:  # to estimate flux from convolved spectra
+        lmin,lmax = [],[]
+        if len(windows)>0:
+            lmin.append(np.amin(np.array([[i[0],i[1]] for i in windows]).flatten()))
+            lmax.append(np.amax(np.array([[i[0],i[1]] for i in windows]).flatten()))
+        if len(ratio_windows)>0:
+            lmin.append(np.amin(np.array([[i[0],i[1]] for i in ratio_windows]).flatten()))
+            lmax.append(np.amax(np.array([[i[0],i[1]] for i in ratio_windows]).flatten()))
+        lmin = np.amin(lmin)
+        lmax = np.amax(lmax)
+        slab_data.convolve(R=2000,lambda_0=lmin*0.9,lambda_n=lmax*1.1,NLTE=NLTE,verbose=False)
+        if NLTE:
+            Fmodel = spectral_fluxes(windows,slab_data.convWavelength,slab_data.convNLTEflux*1e23)
+            Rmodel = spectral_flux_ratios(ratio_windows,slab_data.convWavelength,slab_data.convNLTEflux*1e23)
+        else:
+            Fmodel = spectral_fluxes(windows,slab_data.convWavelength,slab_data.convLTEflux*1e23)
+            Rmodel = spectral_flux_ratios(ratio_windows,slab_data.convWavelength,slab_data.convLTEflux*1e23)
+    else:  # to estimate flux from line list
+        if NLTE:
+            t = 'FNLTE'
+        else:
+            t = 'FLTE'
+        linedata = slab_data.linedata
+        linedata.sort_values(by=['GHz'], ascending=False, inplace=True, ignore_index=True)
+        Fmodel = line_fluxes(windows,c/linedata['GHz']*1e-3,linedata[t])
+        Rmodel = line_flux_ratios(ratio_windows,c/linedata['GHz']*1e-3,linedata[t])
+    Fobserved = spectral_fluxes(windows,spectra[:,0],spectra[:,1])
+    Robserved = spectral_flux_ratios(ratio_windows,spectra[:,0],spectra[:,1])
+    norm_factor = 0
+    if len(windows)>0:
+        area_2d      = np.dstack([area for i in range(len(windows))]).squeeze().reshape((len(area),len(windows)))
+        Fmodel_2d    = np.dstack([Fmodel for i in range(len(area))]).squeeze().T.reshape((len(area),len(windows)))
+        Fobserved_2d = np.dstack([Fobserved for i in range(len(area))]).squeeze().T.reshape((len(area),len(windows)))
+        if len(windows[0])==3: #weighted chi2
+            windows_weights    = np.array([windows[k][2] for k in range(len(windows))])
+            windows_weights_2d = np.dstack([windows_weights for i in range(len(area))]).squeeze().T.reshape((len(area),len(windows_weights)))
+            chi_area          += np.sum(((Fmodel_2d*area_2d-Fobserved_2d)/Fobserved_2d)**2*windows_weights_2d,axis=1)
+            norm_factor       += np.sum(windows_weights)
+        else:
+            chi_area      += np.sum(((Fmodel_2d*area_2d-Fobserved_2d)/Fobserved_2d)**2,axis=1)
+            norm_factor   += len(windows)
+    if len(ratio_windows)>0: 
+        Rmodel_2d    = np.dstack([Rmodel for i in range(len(area))]).squeeze().T.reshape((len(area),len(ratio_windows)))
+        Robserved_2d = np.dstack([Robserved for i in range(len(area))]).squeeze().T.reshape((len(area),len(ratio_windows)))
+        if (len(ratio_windows[0])==3): #weighted chi2
+            ratio_windows_weights    = np.array([ratio_windows[k][2] for k in range(len(ratio_windows))])
+            ratio_windows_weights_2d = np.dstack([ratio_windows_weights for i in range(len(area))]).squeeze().T.reshape((len(area),len(ratio_windows_weights)))
+            chi_area                += np.sum(((Rmodel_2d-Robserved_2d)/Robserved_2d)**2*ratio_windows_weights_2d,axis=1)
+            norm_factor             += np.sum(ratio_windows_weights)
+        else:
+            chi_area      += np.sum(((Rmodel_2d-Robserved_2d)/Robserved_2d)**2,axis=1)
+            norm_factor   += len(ratio_windows)
+    if norm_factor>0: chi_area/=norm_factor
+    return(chi_area)
\ No newline at end of file
