@@ -33,11 +33,15 @@ def main(args=None):
   # Command line parsing
   # this is optional you do not have to do it this way.
   # You can use the prodimopy module in any way you want
-  parser=argparse.ArgumentParser(description='Changing of parameters in the ProDiMo Parameter files. ')
-  parser.add_argument('paramname',help='The name of the Parameter in Parameter.in.')
-  parser.add_argument('value',help='The value of the parameter to change. If the value==DELETEPARAM, the parameter will be deleted')
-  parser.add_argument('-direxp',default=".",help="Change Parameters in all the given model directories. Just pass a regex that is understood by glob.glob")
-  parser.add_argument('-file',required=False,default="Parameter.in",help='File path to the Parameter file. DEFAULT: Parameter.in')
+  parser=argparse.ArgumentParser(description='Changing of parameters in the ProDiMo Parameter files. The changes can usually not be easily undone, so use with care and make a backup.')
+  parser.add_argument('paramname',
+                      help='The name of the Parameter in Parameter.in.')
+  parser.add_argument('value',
+                      help='The value of the parameter to change. If the value==DELETEPARAM, the parameter will be deleted')
+  parser.add_argument('-direxp',default=".",
+                      help="Change Parameters in all the given model directories. Just pass a regex that is understood by glob.glob. To change all Parameter.in files in the current and all subdirecories use './**'.")
+  parser.add_argument('-file',required=False,default="Parameter.in",
+                      help='File path to the Parameter file. DEFAULT: Parameter.in')
   args=parser.parse_args()
 
   print("paramname: ",args.paramname)
@@ -49,10 +53,12 @@ def main(args=None):
   param="! "+args.paramname.strip()+" "
 
   # get the directories
-  dirs=glob.glob(args.direxp)
+  dirs=glob.glob(args.direxp,recursive=True)
   print("")
 
   for dirname in dirs:
+    # Check if files exists, if not just ignore it
+    if not os.path.exists(dirname+"/"+args.file): continue
     print("Change Parameter file: "+dirname+"/"+args.file)
     f=open(dirname+"/"+args.file)
     inlines=f.readlines()
